/**
 * Created by zava on 7.6.2015. https://github.com/filipzava/leaflet-control-bar
 * updated by syonfox on 3.9.2020 https://gitlab.com/syonfox/leaflet-control-bar
 */
L.Control.Bar = L.Control.extend({

    includes: L.Mixin.Events,

    options: {
        position: 'top',
        visible: 'true',
        resizable: 'false',
        height: 250,
        handleHeight: 20,
        handleHTML: ''
    },

    initialize: function (placeholder, options) {
        L.setOptions(this, options);

        // Find content container
        var content = this._contentContainer = L.DomUtil.get(placeholder);


        this.offsets = {
            leafletControl: {
                top: 5,
                bottom: 5,
                left: 5,
                right: 5,
            },

            positionShow: {
                top: 0,
                bottom: 0,
                left: 0,
                right: 0,
            },
            positionHide: {
                top: 5,
                bottom: 5,
                left: 5,
                right: 5,
            },
            height: {
                top: 5,
                bottom: 5,
                left: 5,
                right: 5,
            }
        }

        // this._height = this.options.height;
        // Remove the content container from its original parent
        content.parentNode.removeChild(content);

        // var visibileClass = (this.options.visible) ? 'visible' : '';
        // this.visible = this.options.visible;

        // Create sidebar container
        var container = this._container =
            L.DomUtil.create('div', 'leaflet-control-bar-' + this.options.position + ' leaflet-control-bar ');


        // this.resize(this.options.height)

        this._setHeight(this.options.height);

        if (this.options.visible) {
            this.show();
        } else {
            this.hide();
        }

        // Style and attach content container
        L.DomUtil.addClass(content, 'leaflet-control');

        if (this.options.resizable) {

            switch (this.options.position) {
                case "top": {
                    let handle = this._resizeHandle = L.DomUtil.create('div', 'leaflet-control-bar-handle');
                    handle.innerHTML = this.options.handleHTML;
                    handle.style.height = this.options.handleHeight + 'px';
                    handle.style.minHeight = this.options.handleHeight + 'px';
                    handle.style.width = '100%';

                    container.appendChild(content);
                    container.appendChild(handle);
                    break;
                }
                case "bottom": {
                    let handle = this._resizeHandle = L.DomUtil.create('div', 'leaflet-control-bar-handle');
                    handle.innerHTML = this.options.handleHTML;
                    handle.style.height = this.options.handleHeight + 'px';
                    handle.style.minHeight = this.options.handleHeight + 'px';
                    handle.style.width = '100%';

                    container.appendChild(handle);
                    container.appendChild(content);

                    break;
                }
                case "left": {
                    let handle = this._resizeHandle = L.DomUtil.create('div', 'leaflet-control-bar-handle');
                    handle.innerHTML = this.options.handleHTML;
                    handle.style.height = '100%';
                    handle.style.width = this.options.handleHeight + 'px';
                    handle.style.minWidth = this.options.handleHeight + 'px';

                    container.appendChild(content);
                    container.appendChild(handle);
                    break;
                }
                case "right": {
                    let handle = this._resizeHandle = L.DomUtil.create('div', 'leaflet-control-bar-handle');
                    handle.innerHTML = this.options.handleHTML;
                    handle.style.height = '100%';
                    handle.style.width = this.options.handleHeight + 'px';
                    handle.style.minWidth = this.options.handleHeight + 'px';

                    container.appendChild(handle);
                    container.appendChild(content);
                    break;
                }

            }

        } else {
            container.appendChild(content);
        }


    },

    addTo: function (map) {
        let container = this._container;
        let content = this._contentContainer;
        L.DomEvent
            .on(container, 'transitionend',
                this._handleTransitionEvent, this)
            .on(container, 'webkitTransitionEnd',
                this._handleTransitionEvent, this);

        //

        if (this.options.resizable) {
            let resizeHandle = this._resizeHandle;
            // resizeHandle.innerHTML = '<=====-^-=====>'
            // document.body.appendChild(resizeHandle)
            this._moved = false;
            let self = this
            this._downListener = (e) => {
                console.log('mousedown')
                self._moved = false
                self._mousedown = true;

                self._map.on('mousemove', self._moveListener)
                self._map.once('mouseup', self._upListener)

            }
            resizeHandle.addEventListener('mousedown', this._downListener)
            this._moveListener = (e) => {
                console.log('move')
                console.log(e);

                if (self._mousedown) {

                    let height;
                    switch (this.options.position) {
                        case "top": {
                            height = e.containerPoint.y;
                            break;
                        }
                        case "bottom": {
                            height = self._map._container.clientHeight - e.containerPoint.y;
                            break;
                        }
                        case "left": {
                            height = e.containerPoint.x;
                            break;
                        }
                        case "right": {
                            height = self._map._container.clientWidth - e.containerPoint.x;
                            break;
                        }

                    }
                    self.resize(height);
                }
                self._moved = true
                // self._map.once('mouseup', self._upListener)
            }
            // resizeHandle.addEventListener('mousemove', this._moveListener)
            this._upListener = (e) => {
                self._mousedown = false;

                if (self._moved) {
                    console.log('moved')
                    console.log(e);
                } else {
                    console.log('not moved')
                }

                //this forces content to update
                window.dispatchEvent(new Event('resize'));

                self._map.off('mousemove', this._moveListener)

            }
            // resizeHandle.addEventListener('mouseup', this._upListener)
        }

        var controlContainer = map._controlContainer;
        controlContainer.insertBefore(container, controlContainer.firstChild);

        this._map = map;

        // Make sure we don't drag the map when we interact with the content
        var stop = L.DomEvent.stopPropagation;
        L.DomEvent
            .on(content, 'contextmenu', stop)
            .on(content, 'click', stop)
            .on(content, 'mousedown', stop)
            .on(content, 'touchstart', stop)
            .on(content, 'dblclick', stop)
            .on(content, 'mousewheel', stop)
            .on(content, 'MozMousePixelScroll', stop);

        return this;
    },

    removeFrom: function (map) {
        //if the control is visible, hide it before removing it.
        this.hide();

        var content = this._contentContainer;

        // Remove sidebar container from controls container
        var controlContainer = map._controlContainer;
        controlContainer.removeChild(this._container);

        //disassociate the map object
        this._map = null;

        // Unregister events to prevent memory leak
        var stop = L.DomEvent.stopPropagation;
        L.DomEvent
            .off(content, 'contextmenu', stop)
            .off(content, 'click', stop)
            .off(content, 'mousedown', stop)
            .off(content, 'touchstart', stop)
            .off(content, 'dblclick', stop)
            .off(content, 'mousewheel', stop)
            .off(content, 'MozMousePixelScroll', stop);

        L.DomEvent
            .off(container, 'transitionend',
                this._handleTransitionEvent, this)
            .off(container, 'webkitTransitionEnd',
                this._handleTransitionEvent, this);


        // release memory
        if (this.options.resizable) {
            let resizeHandle = this._resizeHandle;
            resizeHandle.removeEventListener('mousedown', this._downListener)
            resizeHandle.removeEventListener('mousemove', this._moveListener)
            resizeHandle.removeEventListener('mouseup', this._upListener)
        }
        return this;
    },

    isVisible: function () {
        return L.DomUtil.hasClass(this._container, 'visible');
    },

    show: function () {
        if (!this.isVisible()) {
            L.DomUtil.addClass(this._container, 'visible');

            let bottoms = document.getElementsByClassName(`leaflet-${this.options.position}`);
            for (let i = 0; i < bottoms.length; i++) {
                bottoms[i].style.transition = `${this.options.position} 0.5s`;
                bottoms[i].style[this.options.position] = this._height + this.offsets.leafletControl[this.options.position] + 'px';
            }
            this._container.style[this.options.position] = this.offsets.positionShow[this.options.position] + 'px';

            this.fire('show');
        }
    },

    hide: function (e) {
        if (this.isVisible()) {
            L.DomUtil.removeClass(this._container, 'visible');
            this.fire('hide');


            let bottoms = document.getElementsByClassName(`leaflet-${this.options.position}`);
            for (let i = 0; i < bottoms.length; i++) {
                bottoms[i].style.transition = `${this.options.position} 0.5s`;
                bottoms[i].style[this.options.position] = 0 + 'px';
            }
            this._container.style[this.options.position] = -this._height - this.offsets.positionHide[this.options.position] + 'px';

        }
        if (e) {
            L.DomEvent.stopPropagation(e);
        }

    }
    ,

    resize: function (height) {
        // let self = this;
        this.fire('resize');
        this._setHeight(height);
        // this._container.style.height = height + 5 + 'px';
        let bottoms = document.getElementsByClassName(`leaflet-${this.options.position}`);
        for (let i = 0; i < bottoms.length; i++) {
            bottoms[i].style.transition = `${this.options.position} 0s`;
            bottoms[i].style[this.options.position] = height + this.offsets.leafletControl[this.options.position] + 'px';
            // bottoms[i].style.transition = 'bottom 0.5s';

        }
        this.fire('resized');
    }
    ,


    toggle: function () {
        if (this.isVisible()) {
            this.hide();
        } else {
            this.show();
        }
    }
    ,

    getContainer: function () {
        return this._contentContainer;
    }
    ,

    setContent: function (content) {
        this.getContainer().innerHTML = content;
        return this;
    },
    _updateLeafletControls: function(){
        //todo break out the controle update into its own function
    },
    _setHeight: function (height) {
        if (this.options.position === 'top' || this.options.position === 'bottom') {
            this._container.style.height = height + this.offsets.height[this.options.position] + 'px';
        } else {
            this._container.style.width = height + this.offsets.height[this.options.position] + 'px';
        }
        this._height = height;
    }
    ,
    _handleTransitionEvent: function (e) {
        if (e.propertyName === 'left' || e.propertyName === 'right' || e.propertyName === 'bottom' || e.propertyName === 'top')
            this.fire(this.isVisible() ? 'shown' : 'hidden');
    }
});

L.control.bar = function (placeholder, options) {
    return new L.Control.Bar(placeholder, options);
};